﻿using System.Collections.Generic;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 히트 테스트 결과 리스트
        /// </summary>
        private List<DependencyObject> histTestResultList = new List<DependencyObject>();

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            MouseDown += Window_MouseDown;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 윈도우 마우스 이동시 처리하기 - Window_MouseDown(sender, e)

        /// <summary>
        /// 윈도우 마우스 이동시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Window_MouseDown(object sender, MouseButtonEventArgs e)
        {
            Point mousePoint = e.GetPosition(this.canvas);

            HitTestResult result = VisualTreeHelper.HitTest(this.canvas, mousePoint);

            this.histTestResultList.Clear();

            VisualTreeHelper.HitTest
            (
                this.canvas,
                new HitTestFilterCallback(ProcessHitTestFilterCallback),
                new HitTestResultCallback(ProcessHitTestResultCallback),
                new PointHitTestParameters(mousePoint)
            );

            this.textBlock.Text = $"비주얼 히트 수 : {this.histTestResultList.Count}";
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 히트 테스트 필터 콜백 처리하기 - ProcessHitTestFilterCallback(d)

        /// <summary>
        /// 히트 테스트 필터 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <returns>히트 테스트 필터 동작</returns>
        public HitTestFilterBehavior ProcessHitTestFilterCallback(DependencyObject d)
        {
            if(d is Rectangle rectangle)
            {
                if(rectangle.Fill == Brushes.Green)
                {
                    return HitTestFilterBehavior.ContinueSkipSelfAndChildren;
                }
                else
                {
                    return HitTestFilterBehavior.Continue;
                }
            }
            else
            {
                return HitTestFilterBehavior.Continue;
            }
        }

        #endregion
        #region 히트 테스트 결과 콜백 처리하기 - ProcessHitTestResultCallback(result)

        /// <summary>
        /// 히트 테스트 결과 콜백 처리하기
        /// </summary>
        /// <param name="result">히트 테스트 결과</param>
        /// <returns>히트 테스트 결과 동작</returns>
        public HitTestResultBehavior ProcessHitTestResultCallback(HitTestResult result)
        {
            this.histTestResultList.Add(result.VisualHit);

            return HitTestResultBehavior.Continue;
        }

        #endregion
    }
}